{% load i18n %}
{% load model_translation %}
<div class="card">
    <div class="card-header">
        <h3 class="card-title">{% trans "Orginal Fields" %}</h3>
    </div>
    <div class="card-body">
        {% autoescape off %}
            <dl>
                {% for field in object.translatable %}
                    <dt>{% trans field %}</dt>
                    <dd>{{ object|get_field_original_translation:field }}</dd>
                {% endfor %}
            </dl>
        {% endautoescape %}
    </div>
</div>
